package tunnel;

import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;

import java.net.HttpURLConnection;
import java.net.Socket;
import java.net.URL;

public class TunnelConnection implements Runnable {

    private final URL endpoint;

    private final InputStream clientIn;

    private final OutputStream clientOut;

    private String cookie;

    public TunnelConnection(Socket client, URL endpoint) throws IOException {
        this.endpoint = endpoint;
        clientIn = client.getInputStream();
        clientOut = client.getOutputStream();
    }

    private void deliver(byte[] data, int index, int size, OutputStream output)
            throws IOException {
        if (data == null) {
            data = new byte[0];
            index = 0;
            size = 0;
        }
        HttpURLConnection connection = (HttpURLConnection)
                endpoint.openConnection();
        connection.setRequestMethod("POST");
        connection.setDoOutput(true);
        connection.setInstanceFollowRedirects(false);
        connection.setRequestProperty("Accept-Transfer", "");
        connection.setRequestProperty("Content-Type",
                "application/octet-stream");
        connection.setRequestProperty("Content-Length",
                String.valueOf(size));
        synchronized (this) {
            if (cookie != null) connection.setRequestProperty("Cookie", cookie);
            connection.connect();
            OutputStream serverOut = connection.getOutputStream();
            serverOut.write(data, index, size);
            serverOut.flush();
            if (connection.getResponseCode() >= 400) throw new IOException();
            String cookie = connection.getHeaderField("Set-Cookie");
            if (cookie != null) {
                index = cookie.indexOf(';');
                if (index != -1) cookie = cookie.substring(0, index);
                this.cookie = cookie;
            }
        }
        InputStream input = connection.getInputStream();
        data = new byte[8192];
        while ((size = input.read(data, 0, 8192)) != -1) {
            output.write(data, 0, size);
        }
        output.flush();
    }

    public void run() {
        (new Thread() {
            public void run() {
                try {
                    // hack to poll while the connection is open
                    // (to get asynchronously delivered packets).
                    while (true) {
                        deliver(null, 0, 0, clientOut);
                        try {
                            Thread.sleep(1000);
                        } catch (Exception ex) { }
                    }
                } catch (IOException ex) {
                } finally {
                    try {
                        clientIn.close();
                    } catch (Exception ignore) { }
                }
            }
        }).start();
        byte[] buffer = new byte[65536];
        int count;
        try {
            while ((count = clientIn.read(buffer, 0, 65536)) != -1) {
                deliver(buffer, 0, count, clientOut);
            }
        } catch (IOException ex) {
        } finally {
            try {
                clientIn.close();
            } catch (Exception ignore) { }
        }
    }

}
