package tunnel;

import java.io.IOException;

import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.URL;

public class TunnelListener extends Thread {

    private final ServerSocket listener;

    private final URL endpoint;

    public TunnelListener(String endpoint, int port, boolean external)
            throws IOException {
        this(new URL(endpoint), port, external);
    }

    public TunnelListener(URL endpoint, int port, boolean external)
            throws IOException {
        this.endpoint = endpoint;
        listener = external ? new ServerSocket(port) :
                new ServerSocket(port, 0, InetAddress.getByName(null));
    }

    public void run() {
        try {
            while (true) {
                TunnelConnection connection =
                        new TunnelConnection(listener.accept(), endpoint);
                new Thread(connection).start();
            }
        } catch (IOException ex) {
            System.err.println("listener exception: " + ex);
            ex.printStackTrace();
        }
    }

    public static void main(String[] args) throws Exception {
        // remote endpoint, i.e. http://server/smbtarget/
        String endpoint = args[0];
        int port = Integer.parseInt(args[1]);
        TunnelListener l = new TunnelListener(endpoint, 139, false);
        l.start();
        System.out.println("Listening for SMB requests on port " + port);
        Object o = new Object();
        synchronized (o) {
            // wait forever
            o.wait();
        }
    }

}
