package tunnel;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;

import java.net.Socket;

import java.util.HashMap;
import java.util.Map;

public class TunnelPipe {

    private static final Map PIPES = new HashMap();

    private final String pipeId;

    private final InputStream pipeIn;

    private final OutputStream pipeOut;

    public static TunnelPipe getInstance(String host, int port,
            String clientId) throws IOException {
        String pipeId = host + ":" + port + ":" + clientId;
        synchronized (PIPES) {
            TunnelPipe pipe = (TunnelPipe) PIPES.get(pipeId);
            if (pipe != null) return pipe;
            Socket socket = new Socket(host, port);
            pipe = new TunnelPipe(pipeId, socket.getInputStream(),
                    socket.getOutputStream());
            PIPES.put(pipeId, pipe);
            return pipe;
        }
    }

    protected TunnelPipe(String pipeId, InputStream pipeIn,
            OutputStream pipeOut) {
        this.pipeId = pipeId;
        this.pipeIn = pipeIn;
        this.pipeOut = pipeOut;
        if (pipeId == null || pipeIn == null || pipeOut == null) {
            throw new NullPointerException();
        }
    }

    public void deliver(int count, InputStream in, OutputStream out)
            throws IOException {
        if (count != 0) {
            final byte[] data;
            if (count != -1) {
                data = new byte[count];
                in.read(data, 0, count);
            } else {
                byte[] buffer = new byte[1024];
                ByteArrayOutputStream collector =
                        new ByteArrayOutputStream(1024);
                while ((count = in.read(buffer, 0, 1024)) != -1) {
                    collector.write(buffer, 0, count);
                }
                data = collector.toByteArray();
            }
            (new Thread() {
                public void run() {
                    try {
                        synchronized (pipeOut) {
                            pipeOut.write(data);
                            pipeOut.flush();
                        }
                    } catch (IOException ex) {
                        System.err.println("error pumping to dest: " + ex);
                        ex.printStackTrace();
                    }
                }
            }).start();
        }
        byte[] buffer = new byte[1024];
        synchronized (pipeIn) {
            while ((count = Math.min(1024, pipeIn.available())) != 0 &&
                    (count = pipeIn.read(buffer, 0, (count != -1) ?
                            count : 1024)) != -1) {
                out.write(buffer, 0, count);
            }
        }
        out.flush();
        if (count == -1) System.err.println("<< end of pipe reached >>");
    }

}
